// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.
'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const types_1 = require("../../../common/application/types");
const logger_1 = require("../../../common/logger");
// import { swallowExceptions } from '../../../common/utils/decorators';
const misc_1 = require("../../../common/utils/misc");
const eventName = 'ptvsd_subprocess';
let ChildProcessLaunchEventHandler = class ChildProcessLaunchEventHandler {
    constructor(appShell, debugService, workspaceService) {
        this.appShell = appShell;
        this.debugService = debugService;
        this.workspaceService = workspaceService;
    }
    // @swallowExceptions('Handle child process launch')
    handleEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!event || event.event !== eventName) {
                return;
            }
            const data = event.body;
            const folder = this.getRelatedWorkspaceFolder(data);
            const debugConfig = this.getAttachConfiguration(data);
            const launched = yield this.debugService.startDebugging(folder, debugConfig);
            if (!launched) {
                this.appShell.showErrorMessage(`Failed to launch debugger for child process ${data.processId}`).then(misc_1.noop, misc_1.noop);
            }
        });
    }
    getRelatedWorkspaceFolder(_data) {
        return this.workspaceService.workspaceFolders[0];
    }
    getAttachConfiguration(data) {
        // tslint:disable-next-line:no-any
        const config = JSON.parse(JSON.stringify(data.rootStartRequest.arguments));
        if (data.rootStartRequest.arguments.request === 'attach') {
            config.host = data.rootStartRequest.arguments.host;
        }
        config.port = data.port;
        config.name = `Child Process ${data.processId}`;
        config.request = 'attach';
        return config;
    }
};
__decorate([
    logger_1.traceError('Handle child process launch')
], ChildProcessLaunchEventHandler.prototype, "handleEvent", null);
ChildProcessLaunchEventHandler = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(types_1.IApplicationShell)),
    __param(1, inversify_1.inject(types_1.IDebugService)),
    __param(2, inversify_1.inject(types_1.IWorkspaceService))
], ChildProcessLaunchEventHandler);
exports.ChildProcessLaunchEventHandler = ChildProcessLaunchEventHandler;
//# sourceMappingURL=childProcessLaunchHandler.js.map